/**
 * js logic specific to opening and closing the panels from the site header elements
 */

import ee from "./event-emitter";
import { ariaToggle, toArray } from "./dom-methods";

const panelSelector = "#o-panel";
const panelCatSelector = ".o-panel-categories";
const panelSearchSelector = ".o-panel-search";
const panelBgHitSelector = ".o-panel-bg-area";
const menuButtonSelector = ".c-control--menu-button";
const searchButtonSelector = ".c-control--search-button";
const closeControlSelector = ".c-control--close";
const panelOpenedClass = "o-panel--opened";
const panelOpeningClass = "o-panel--opening";
const panelClosingClass = "o-panel--closing";
const panelCatsOpenClass = "o-panel-categories--opened";
const panelSearchOpenClass = "o-panel-search--opened";
const animationTime = 250;
const delayClassSwitchTime = 50;

let menuButton = null;
let panel = null;
let panelCategories = null;
let panelSearch = null;
let panelBgHit = null;
let closeControl = null;
let searchButton = null;
let classSwapTimeout = null;
let delayTimeout = null;

/**
 * @method init
 * @description init method, adds event listener for scroll and resize
 * @return {void}
 */
export default function init() {
  panel = document.querySelector(panelSelector);
  panelCategories = document.querySelector(panelCatSelector);
  panelSearch = document.querySelector(panelSearchSelector);
  menuButton = toArray(document.querySelectorAll(menuButtonSelector));
  searchButton = toArray(document.querySelectorAll(searchButtonSelector));
  closeControl = toArray(document.querySelectorAll(closeControlSelector));

  createPanelListeners();
}

/**
 * @method createPanelListeners
 * @description adds event listeners to panel buttons
 * @return {void}
 */
export function createPanelListeners() {
  menuButton.forEach((el) => {
    el.addEventListener("click", onCategoriesClicked);
  });

  closeControl.forEach((el) => {
    el.addEventListener("click", panelClose);
  });

  searchButton.forEach((el) => {
    el.addEventListener("click", onSearchClicked);
  });
}

/**
 * @method removePanelListeners
 * @description adds event listeners to panel buttons
 * @return {void}
 */
export function removePanelListeners() {
  menuButton.forEach((el) => {
    el.removeEventListener("click", onCategoriesClicked);
  });

  closeControl.forEach((el) => {
    el.removeEventListener("click", panelClose);
  });

  searchButton.forEach((el) => {
    el.removeEventListener("click", onSearchClicked);
  });
}

/**
 * @method onCategoriesClicked
 * @description on menu clicked
 * @return {void}
 */
export function onCategoriesClicked() {
  const isMenuOpen = panel.classList.contains(panelOpenedClass) || panel.classList.contains(panelOpeningClass);
  const isCatOpen = panelCategories.classList.contains(panelCatsOpenClass);

  isMenuOpen && isCatOpen ? panelClose() : categoriesOpen();
}

/**
 * @method onSearchClicked
 * @description adds open class
 * @return {void}
 */
export function onSearchClicked() {
  const isMenuOpen = panel.classList.contains(panelOpenedClass) || panel.classList.contains(panelOpeningClass);
  const isSearchOpen = panelSearch.classList.contains(panelSearchOpenClass);

  isMenuOpen && isSearchOpen ? panelClose() : searchOpen();
}

/**
 * @method categoriesOpen
 * @description adds open class
 * @return {void}
 */
export function categoriesOpen() {
  openPanel("categories");
}

/**
 * @method searchOpen
 * @description adds open class
 * @return {void}
 */
export function searchOpen() {
  openPanel("search");
}

/**
 * @method openPanel
 * @description opens the panel and dynamically emits the component name for analytics tracking purposes
 * @param {string} [componentName = categories] one of two string values for opening the panel: "categories" or "search"
 * @return {void}
 */
export function openPanel(componentName = "categories") {
  panel.removeAttribute("hidden");

  clearTimeout(delayTimeout);
  delayTimeout = setTimeout(() => {
    setPanelTransitionClass(panelOpeningClass);
    if (componentName === "categories") {
      panelCategories.classList.add(panelCatsOpenClass);
      panelSearch.classList.remove(panelSearchOpenClass);
    } else {
      panelSearch.classList.add(panelSearchOpenClass);
      panelCategories.classList.remove(panelCatsOpenClass);
    }

    ariaToggle(".c-control--menu-button .c-control__button", `close ${componentName}`, true);
    navigationFade();

    ee.emit(`panel:${componentName}-opening`);
    ee.emit("panel:opening");
  }, delayClassSwitchTime);

  clearTimeout(classSwapTimeout);
  classSwapTimeout = setTimeout(() => {
    setPanelTransitionClass(panelOpenedClass);
    createTransparentClickableBG();
    ee.emit(`panel:${componentName}-opened`);
    ee.emit("panel:opened");
  }, animationTime + delayClassSwitchTime);
}

/**
 * @method panelClose
 * @description removes open classes
 * @return {void}
 */
export function panelClose() {
  setPanelTransitionClass(panelClosingClass);

  ariaToggle(".c-control--menu-button .c-control__button", "open navigation", false);
  ariaToggle(".c-control--search-button .c-control__button", "open search", false);
  navigationUnFade();
  removeBgPanel();

  ee.emit("panel:closing");
  ee.emit("panel:search-closing");
  ee.emit("panel:categories-closing");

  clearTimeout(classSwapTimeout);
  classSwapTimeout = setTimeout(() => {
    setPanelTransitionClass(null);
    panelCategories.classList.remove(panelCatsOpenClass);
    panelSearch.classList.remove(panelSearchOpenClass);

    panel.setAttribute("hidden", "hidden");

    ee.emit("panel:closed");
    ee.emit("panel:search-closed");
    ee.emit("panel:categories-closed");
  }, animationTime);
}

/**
 * @method setPanelTransitionClass
 * @description swaps out panel transition/state class and adds a new class
 * @param {string|null} [className = null] class name to swap in
 * @return {void}
 */
export function setPanelTransitionClass(className = null) {
  panel.classList.remove(panelOpeningClass, panelOpenedClass, panelClosingClass);
  className && panel.classList.add(className);
}

/**
 * @method createTransparentClickableBG
 * @description adds the hidden background element
 * @return {void}
 */
export function createTransparentClickableBG() {
  if (!document.querySelector(".o-panel-bg-area")) {
    const divPanelBG = document.createElement("div");
    divPanelBG.classList.add("o-panel-bg-area");
    document.body.appendChild(divPanelBG);
    panelBgHit = document.querySelector(panelBgHitSelector);
    panelBgHit.addEventListener("click", panelClose);
  } else {
    document.querySelector(panelBgHitSelector) && document.querySelector(panelBgHitSelector).classList.remove("o-panel-bg-area--closed");
  }
}

/**
 * @method removeBgPanel
 * @description removes the hidden background element
 * @return {void}
 */
export function removeBgPanel() {
  document.querySelector(panelBgHitSelector) && document.querySelector(panelBgHitSelector).classList.add("o-panel-bg-area--closed");
}

/**
 * @method getHeaderElement
 * @description gets the current header element
 * @return {HTMLElement}
 */
export function getHeaderElement() {
  return document.querySelector(".c-header-nav") || document.querySelector(".c-header-title--article") || document.querySelector(".c-header-title--section");
}

/**
 * @method navigationFade
 * @description adds fade class to specific elements
 * @return {void}
 */
export function navigationFade() {
  const fadeClass = "c-control--disabled";
  const h2 = document.querySelector("h2");
  const scrollNav = document.querySelector(".c-scroll-menu__nav");
  const header = getHeaderElement();
  const allControlButtons = toArray(header.querySelectorAll(".c-control"));
  const ignoreClasses = ["c-control--menu-button", "c-control--search-button"];

  allControlButtons.forEach((c) => {
    let notAnIgnoredClass = ignoreClasses.every((ignoreString) => {
      return !c.classList.contains(ignoreString);
    });

    notAnIgnoredClass && c.classList.add(fadeClass);
  });

  scrollNav && scrollNav.classList.add(fadeClass);
  h2 && h2.classList.add(fadeClass);
}

/**
 * @method navigationUnFade
 * @description removes fade class to specific elements
 * @return {void}
 */
export function navigationUnFade() {
  const fadeClass = "c-control--disabled";
  const h2 = document.querySelector("h2");
  const scrollNav = document.querySelector(".c-scroll-menu__nav");
  const header = getHeaderElement();
  const allControlButtons = toArray(header.querySelectorAll(".c-control"));

  allControlButtons.forEach((c) => {
    c.classList.remove(fadeClass);
  });

  scrollNav && scrollNav.classList.remove(fadeClass);
  h2 && h2.classList.remove(fadeClass);
}
